export declare type CommonMetadata = {
    description?: string;
    [key: string]: any;
};
export declare type CodecProps = {
    metadata?: CommonMetadata;
    required?: boolean;
    [key: string]: any;
};
export declare type Transformer<I, O> = (data: I) => O;
export declare type Codec<I, O, T = string, P = CodecProps> = {
    _tag: T;
    props: CodecProps & P;
    meta: (metadata: CommonMetadata) => Codec<I, O, T, P>;
    encode: Transformer<I, O>;
    decode: Transformer<O, I>;
    and: <T extends AnyCodec>(codec: T) => Intersection<Codec<I, O>, T>;
    or: <T extends AnyCodec>(codec: T) => Union<Codec<I, O>, T>;
    optional: () => OptionalCodec<Codec<I, O, T, P>>;
};
export declare type PassThroughCodec<T> = Codec<T, T>;
export declare type AnyCodec = Codec<any, any, any>;
export declare type Cx<C extends AnyCodec> = C extends Codec<infer I, infer O, infer T, infer P> ? {
    I: I;
    O: O;
    T: T;
    P: P;
} : never;
export declare type Ix<C extends AnyCodec> = Cx<C>['I'];
export declare type Ox<C extends AnyCodec> = Cx<C>['O'];
export declare type Encoded<T extends AnyCodec> = Ox<T>;
export declare type Decoded<T extends AnyCodec> = Ix<T>;
export declare type AnyObjectCodecShape = Record<string, AnyCodec>;
declare type OptionalKeys<T extends object> = {
    [K in keyof T]: undefined extends T[K] ? K : never;
}[keyof T];
declare type RequiredKeys<T extends object> = Exclude<keyof T, OptionalKeys<T>>;
declare type WithOptional<T extends object> = {
    [K in RequiredKeys<T>]: T[K];
} & {
    [K in OptionalKeys<T>]?: T[K];
};
declare type MappedIx<T extends AnyObjectCodecShape> = WithOptional<{
    [K in keyof T]: Ix<T[K]>;
}>;
declare type MappedOx<T extends AnyObjectCodecShape> = WithOptional<{
    [K in keyof T]: Ox<T[K]>;
}>;
export declare enum CodecType {
    String = "string",
    Number = "number",
    Boolean = "boolean",
    Literal = "literal",
    Enum = "enum",
    Null = "null",
    Any = "any",
    Object = "object",
    Record = "record",
    Array = "array",
    Tuple = "tuple",
    Recursive = "recursive",
    Union = "union",
    Intersection = "intersection"
}
export declare type Intersection<C1 extends AnyCodec, C2 extends AnyCodec> = Codec<Ix<C1> & Ix<C2>, Ox<C1> & Ox<C2>, CodecType.Intersection, {
    codecs: AnyCodec[];
}>;
export declare type Union<C1 extends AnyCodec, C2 extends AnyCodec> = Codec<Ix<C1> | Ix<C2>, Ox<C1> | Ox<C2>, CodecType.Union, {
    codecs: AnyCodec[];
}>;
export declare type ObjectCodec<T extends AnyObjectCodecShape> = Codec<MappedIx<T>, MappedOx<T>, CodecType.Object, {
    shape: T;
}>;
export declare type RecordCodec<T extends AnyCodec> = Codec<Record<string, Ix<T>>, Record<string, Ox<T>>, CodecType.Record, {
    type: T;
}>;
export declare type ArrayCodec<T extends AnyCodec> = Codec<Ix<T>[], Ox<T>[], CodecType.Array, {
    type: T;
}>;
export declare type AnyTuple = [AnyCodec, ...AnyCodec[]];
export declare type AssertArray<T> = T extends any[] ? T : never;
export declare type TupleIx<T extends AnyTuple> = AssertArray<{
    [k in keyof T]: T[k] extends AnyCodec ? Ix<T[k]> : never;
}>;
export declare type TupleOx<T extends AnyTuple> = AssertArray<{
    [k in keyof T]: T[k] extends AnyCodec ? Ox<T[k]> : never;
}>;
export declare type TupleCodec<T extends AnyTuple> = Codec<TupleIx<T>, TupleOx<T>, CodecType.Tuple, {
    codecs: T;
}>;
export declare type RecursiveCodec<T extends AnyCodec> = Codec<Ix<T>, Ox<T>, CodecType.Recursive, {
    id: string;
    resolver: () => T;
}>;
export declare type OptionalCodec<T extends AnyCodec> = Codec<Ix<T> | undefined, Ox<T> | undefined, Cx<T>['T'], Cx<T>['P']>;
declare type IdentityMapping<T extends CodecType> = T extends CodecType.String ? string : T extends CodecType.Number ? number : T extends CodecType.Boolean ? boolean : never;
export declare type IdentityCodec<T extends CodecType> = Codec<IdentityMapping<T>, IdentityMapping<T>, T>;
export declare type LiteralCodec<T extends string> = Codec<T, T, CodecType.Literal, {
    value: T;
}>;
export declare type EnumLike = {
    [key: string]: string;
};
export declare type EnumCodec<T extends EnumLike> = Codec<T[keyof T], T[keyof T], CodecType.Enum, {
    enum: EnumLike;
}>;
export {};
