"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateJSONSchema = exports.createParser = exports.TransformTarget = void 0;
const definitions_1 = require("./definitions");
var TransformTarget;
(function (TransformTarget) {
    TransformTarget["Encoded"] = "encoded";
    TransformTarget["Decoded"] = "decoded";
})(TransformTarget = exports.TransformTarget || (exports.TransformTarget = {}));
const createParser = (tag, parser) => {
    return {
        tag,
        parse: parser
    };
};
exports.createParser = createParser;
const createPrimitiveParser = (type) => {
    return {
        tag: type,
        parse: () => {
            return { type: type };
        }
    };
};
const StringParser = createPrimitiveParser(definitions_1.CodecType.String);
const NumberParser = createPrimitiveParser(definitions_1.CodecType.Number);
const BooleanParser = createPrimitiveParser(definitions_1.CodecType.Boolean);
const NullParser = createPrimitiveParser(definitions_1.CodecType.Null);
const AnyParser = (0, exports.createParser)(definitions_1.CodecType.Any, (codec) => {
    return {};
});
const EnumParser = (0, exports.createParser)(definitions_1.CodecType.Enum, (codec) => {
    return {
        type: 'string',
        enum: Object.values(codec.props.enum)
    };
});
const LiteralParser = (0, exports.createParser)(definitions_1.CodecType.Literal, (codec) => {
    return {
        type: 'string',
        const: codec.props.value
    };
});
const ObjectParser = (0, exports.createParser)(definitions_1.CodecType.Object, (codec, options) => {
    const entries = Object.entries(codec.props.shape);
    return {
        type: 'object',
        properties: entries.reduce((acc, [key, codec]) => {
            acc[key] = RootParser(codec, options);
            return acc;
        }, {}),
        additionalProperties: !!options?.allowAdditional,
        required: entries
            .filter(([, codec]) => {
            return codec.props.required;
        })
            .map(([key]) => key)
    };
});
const RecordParser = (0, exports.createParser)(definitions_1.CodecType.Record, (codec, options) => {
    return {
        type: 'object',
        additionalProperties: RootParser(codec.props.type, options),
        properties: {},
        required: []
    };
});
const ArrayParser = (0, exports.createParser)(definitions_1.CodecType.Array, (codec, options) => {
    return {
        type: 'array',
        items: RootParser(codec.props.type, options)
    };
});
const TupleParser = (0, exports.createParser)(definitions_1.CodecType.Tuple, (codec, options) => {
    return {
        type: 'array',
        items: codec.props.codecs.map((codec) => RootParser(codec, options)),
        minItems: codec.props.codecs.length,
        maxItems: codec.props.codecs.length
    };
});
const IntersectionParser = (0, exports.createParser)(definitions_1.CodecType.Intersection, (codec, options) => {
    const schemas = codec.props.codecs.map((codec) => RootParser(codec, options));
    // return {
    //   type: "object",
    //   properties: schemas.reduce((properties, schema: any) => {
    //     return {
    //       ...properties,
    //       ...(schema.properties || {}),
    //     }
    //   }, {}),
    //   additionalProperties: !!options?.allowAdditional,
    //   required: schemas.reduce((required: string[], schema: any) => {
    //     return required.concat(schema.required || [])
    //   }, []),
    // }
    return {
        allOf: schemas
    };
});
const UnionParser = (0, exports.createParser)(definitions_1.CodecType.Union, (codec, options) => {
    return {
        anyOf: codec.props.codecs.map((codec) => RootParser(codec, options))
    };
});
const RecursiveParser = (0, exports.createParser)(definitions_1.CodecType.Recursive, (codec, options) => {
    const ref = {
        $ref: `#/definitions/${codec.props.id}`
    };
    const cached = options.cache.get(codec.props.id);
    if (cached) {
        return ref;
    }
    options.cache.set(codec.props.id, {});
    /**
     * The schema is applied to the definition _after_ pushing the definition to the cache
     * to ensure that there are no cache misses from calling RootParser
     */
    options.cache.set(codec.props.id, RootParser(codec.props.resolver(), options));
    return ref;
});
const RootParser = (codec, options) => {
    const parser = options.parsers.find((parser) => parser.tag === codec._tag);
    if (!parser) {
        throw new Error(`No parser configured for codec ${codec._tag}`);
    }
    const schema = parser.parse(codec, options);
    if (codec.props.metadata?.description) {
        return {
            description: codec.props.metadata.description,
            ...schema
        };
    }
    return schema;
};
const generateJSONSchema = (codec, options) => {
    const parsers = [
        ...(options?.parsers || []),
        AnyParser,
        StringParser,
        NumberParser,
        BooleanParser,
        NullParser,
        LiteralParser,
        EnumParser,
        ObjectParser,
        RecordParser,
        ArrayParser,
        TupleParser,
        IntersectionParser,
        UnionParser,
        RecursiveParser
    ];
    const recursion_cache = new Map();
    const schema = RootParser(codec, {
        parsers,
        target: options?.target ?? TransformTarget.Encoded,
        allowAdditional: options?.allowAdditional ?? false,
        cache: recursion_cache
    });
    return {
        definitions: Object.fromEntries(recursion_cache.entries()),
        ...schema
    };
};
exports.generateJSONSchema = generateJSONSchema;
//# sourceMappingURL=generation.js.map